
/*
 * This file is used to write data to a portable pixel map file and I copied
 * it from the ucalgary CPSC jungle website .... Ahmed Obied ....
 */

#include <string.h>
#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include <GL/gl.h>
#include "pixelmap.h"

#define clamp(x,a,b) ((x)<(a) ? (a) : ((x)>(b) ? (b) : (x)))
#define min(a,b) ((a)<(b) ? (a) : (b))

PixelMap :: PixelMap( FILE* fil) {
  map=0;
  load(fil);
}

PixelMap :: PixelMap(const char* frame) {
  map=0;
  load(frame);
}

PixelMap :: PixelMap(GLint _w, GLint _h) {
  w= _w, h= _h;
  map=new GLubyte[w*h*3];
}

PixelMap :: PixelMap(const PixelMap* m) {
  w= m->w, h= m->h;
  memcpy(map,m->map,w*h*3);
}

PixelMap :: ~PixelMap() {
  delete [] map;
}

void PixelMap :: load(FILE* fil) {
  if (map) delete[] map;
  if (fil) {
    if (!readHeader(fil)) {
            cout << "ppm Error:  invalid pixmap file."
		  << endl;
	  } else {
	    map=new GLubyte[w*h*3];
	    fread(map,1,w*h*3,fil);
	  }
  } else {
    cout << "ppm Error: cannot open file."
      << endl;
  }
}

void PixelMap :: load(const char* fname) {
  if (map) delete[] map;
  FILE* fil = fopen(fname,"rb");
  if (fil) {
    if (!readHeader(fil)) {
            cout << "ppm Error: "
	      << fname
		<< " is not a valid pixmap file."
		  << endl;
	  } else {
	    map=new GLubyte[w*h*3];
	    fread(map,1,w*h*3,fil);
	  }
    fclose(fil);
  } else {
    cout << "ppm Error: "
      << fname
	<< " does not exist."
	  << endl;
  }
}

void PixelMap :: reset(GLint _w, GLint _h) {
  delete [] map;
  w = _w, h = _h;
  map = new GLubyte[w*h*3];
}

const GLubyte* PixelMap :: pixel(GLint x, GLint y) const {
  x = clamp(x,0,w-1), y = clamp(y,0,h-1);
  return &map[y*w*3+x*3];
}

GLubyte PixelMap :: red(GLint x, GLint y) const {
  return pixel(x,y)[0];
}

GLubyte PixelMap :: green(GLint x, GLint y) const {
  return pixel(x,y)[1];
}

GLubyte PixelMap :: blue(GLint x, GLint y) const {
  return pixel(x,y)[2];
}

GLubyte PixelMap :: component(GLint i, GLint x, GLint y) const {
  return i ? (i==1 ? green(x,y) : blue(x,y)) : red(x,y);
}

void PixelMap :: backGround(GLubyte cr, GLubyte cg, GLubyte cb) {
  for (GLint x=0; x<width(); x++) {
    for (GLint y=0; y<height(); y++) {
	  pixel(x,y,cr,cg,cb);
	}
  }
}

void PixelMap :: checkers(GLubyte cr, GLubyte cg, GLubyte cb) {  
  const GLubyte mask = 255;
  const GLint D = min(height(), width())/16;
  GLubyte r,g,b;
  
  for (GLint x=0; x<width(); x++) {
    for (GLint y=0; y<height(); y++) {
      r=cr; g=cg; b=cb;
      if ((!((x/D)%2) && !((y/D)%2)) || (((x/D)%2) && ((y/D)%2)))
	  r^=mask, g^=mask, b^=mask;
	  pixel(x,y,r,g,b);
	}
  }
}

void PixelMap :: whiteSpace(FILE* f) {
  int c;
  do {
    c=fgetc(f);
    if (c=='#') while ((c=fgetc(f))!='\n');
  }
  while (c==' ' | c==7 || c=='\r' || c=='\n');
  ungetc(c,f);
}

int  PixelMap :: readHeader(FILE* f) {
  int ok=1;
  char s[2];
  fread(s,1,2,f);

  if (s[0]=='P' && s[1]=='6') {
    whiteSpace(f);
    fscanf(f,"%d",&w);
    whiteSpace(f);
    fscanf(f,"%d",&h);    
    whiteSpace(f);
    fscanf(f,"255");
    whiteSpace(f);
  } else ok=0;
  return ok;
}

ostream& operator << (ostream& os, const PixelMap& map) {
  os << "P6" << map.width() << ' ' << map.height() << ' ' << "255\n";
  for (GLint y=0; y<map.height(); y++)
    for (GLint x=0; x<map.width(); x++) {
      const GLubyte * c = map.pixel(x,y);
      os << c[0] << c[1] << c[2];
    }
    return os;
}
