/*
 *  sphere.cpp
 *
 *  Author : Ahmed Obied (ahmed.obied@gmail.com)
 *
 */

#include "sphere.h"
#include "iostream.h"
#include "material.h"

Sphere::Sphere()
{
}

void Sphere::setRadius(double r)
{
  radius = r;
}

void Sphere::setCenter(gmVector3 center)
{
  origin = center;
}

double Sphere::getRadius()
{
  return radius;
}

void Sphere::setMat(Color amb,Color dif,Color spec,
	    double ref, double trans, int shininess)
{
  shapeMat.setAmbient(amb);
  shapeMat.setDiffuse(dif);
  shapeMat.setSpecular(spec);
  shapeMat.setReflection(ref);
  shapeMat.setTransmission(trans);
  shapeMat.setShininess(shininess);
}

gmVector3 Sphere::getCenter()
{
  return origin;
}

Material Sphere::getMat()
{
  return shapeMat;
}

gmVector3 Sphere::intersection(gmVector3 p,gmVector3 dir,int &status)
{
  gmVector3 temp;
  double originArray[3];
  double pArray[3];
  double dirArray[3];
  double a,b,c;
  double t,t_1,t_2;
  double discriminant;
  // calculate intersection point if there is a hit with the sphere
  gmVector3 intersectionPoint;
  status = 0;
  a = dot(dir,dir);
  dir.copyTo(dirArray);
  p.copyTo(pArray);
  origin.copyTo(originArray);
  b = 2*((dirArray[0]*(pArray[0]-originArray[0]))+
	 (dirArray[1]*(pArray[1]-originArray[1]))+
	 (dirArray[2]*(pArray[2]-originArray[2])));
  c = (pow((pArray[0] - originArray[0]),2) +
       pow((pArray[1] - originArray[1]),2) +
       pow((pArray[2] - originArray[2]),2)) 
    - pow(radius*radius,2);
  discriminant = pow(b,2) - (4 * c);
  if (discriminant >  0)
    {
      status = 1;
      t_1 = (-1 * b + discriminant)/2;
      t_2 = (-1 * b - discriminant)/2;
      if (t_1 > 0 && t_2 > 0)
	{
	  if (t_1 < t_2)
	    t = t_1;
	  else
	    t = t_2;
	}
      else
	status = 0;
    }
  else
    status = 0;
  temp = t * dir;
  // return the intersection point with the sphere
  intersectionPoint = p + temp;
  return intersectionPoint;
}

gmVector3 Sphere::getNormalVector(gmVector3 point)
{
  gmVector3 n;
  // calculate normal
  n = point - origin;
  n.normalize();
  // return the normal at that point
  return n;
}

gmVector3 Sphere::getReflectionVector(gmVector3 dir,gmVector3 normal)
{
  gmVector3 rVec;
  gmVector3 temp;
  double temp_1;
  // find the cosine angle
  temp_1 = dot(dir,normal);
  // extend the angle
  temp = -1 * 2 * normal * temp_1;
  // compute the reflection vector and normalize
  rVec = temp + dir;
  rVec.normalize();
  // return the reflection vector
  return rVec;
}

gmVector3 Sphere::getTransmissionVector(gmVector3 normal,
					gmVector3 dir)
{
  gmVector3 tVec;
  return tVec;
  }
